<?php

namespace Drupal\social_auth_strava\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Render\RendererInterface;
use Drupal\social_auth\User\UserAuthenticator;
use Drupal\social_api\Plugin\NetworkManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Session\SessionInterface;
use Drupal\social_auth_strava\StravaAuthManager;
use Drupal\Core\Render\RenderContext;

/**
 * Manages requests to the Strava API.
 */
class StravaAuthController extends ControllerBase {

  /**
   * The network plugin manager.
   *
   * @var \Drupal\social_api\Plugin\NetworkManager
   */
  protected NetworkManager $networkManager;

  /**
   * The Strava authentication manager.
   *
   * @var \Drupal\social_auth_strava\StravaAuthManager
   */
  protected StravaAuthManager $stravaManager;

  /**
   * The user authenticator.
   *
   * @var \Drupal\social_auth\User\UserAuthenticator
   */
  protected UserAuthenticator $userAuthenticator;

  /**
   * The session manager.
   *
   * @var \Symfony\Component\HttpFoundation\Session\SessionInterface
   */
  protected SessionInterface $session;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected RendererInterface $renderer;

  /**
   * GoogleLoginController constructor.
   *
   * @param \Drupal\social_api\Plugin\NetworkManager $network_manager
   *   Used to get an instance of social_auth_strava network plugin.
   * @param \Drupal\social_auth\User\UserAuthenticator $userAuthenticator
   *   Manages user login/registration.
   * @param \Drupal\social_auth_strava\StravaAuthManager $stravaManager
   *   Used to manage authentication methods.
   * @param \Symfony\Component\HttpFoundation\Session\SessionInterface $session
   *   Used to store the access token into a session variable.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   */
  public function __construct(NetworkManager $network_manager, UserAuthenticator $userAuthenticator, StravaAuthManager $stravaManager, SessionInterface $session, RendererInterface $renderer) {
    $this->networkManager = $network_manager;
    $this->userAuthenticator = $userAuthenticator;
    $this->stravaManager = $stravaManager;
    $this->session = $session;
    $this->renderer = $renderer;

    // Sets the plugin id.
    $this->userAuthenticator->setPluginId('social_auth_strava');
    // Sets the session keys to nullify if user could not logged in.
    $this->userAuthenticator->setSessionKeysToNullify(['social_auth_strava_access_token']);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.network.manager'),
      $container->get('social_auth.user_authenticator'),
      $container->get('social_auth_strava.manager'),
      $container->get('session'),
      $container->get('renderer')
    );
  }

  /**
   * Redirect to Strava Services Authentication page.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   The redirect response.
   */
  public function redirectToStrava(): RedirectResponse {
    // We need to avoid rendering too early.
    $context = new RenderContext();
    $client = $this->renderer->executeInRenderContext($context, function () {
      return $this->networkManager
        ->createInstance('social_auth_strava')
        ->getSdk();
    });

    // If Strava client could not be obtained.
    if (!$client) {
      $this->messenger()
        ->addError($this->t('Social Auth Strava not configured properly. Contact site administrator.'));
      return $this->redirect('user.login');
    }

    // Strava service was returned, inject it to $linkedinManager.
    $this->stravaManager->setClient($client);

    return new TrustedRedirectResponse($this->stravaManager->getAuthorizationUrl());
  }

  /**
   * Callback function to log in user.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   The redirect response.
   */
  public function callback(): RedirectResponse {
    $client = $this->networkManager
      ->createInstance('social_auth_strava')
      ->getSdk();

    $this->stravaManager->setClient($client)->authenticate();

    // Saves access token so that event subscribers can call Strava API.
    $this->session->set('social_auth_strava_access_token', $this->stravaManager->getAccessToken());

    // Gets user information.
    $user = $this->stravaManager->getUserInfo();

    // If user information could be retrieved.
    if ($user) {
      $picture = $user->imageUrl ?? FALSE;

      return $this->userAuthenticator->authenticateUser(
        $user->name,
        $user->email,
        $user->uid,
        $picture
      );
    }

    $this->messenger()
      ->addError($this->t('You could not be authenticated, please contact the administrator'));

    return $this->redirect('user.login');
  }

}
